﻿Type=Class
Version=6.5
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Browser customview with designer properties, hidable top bar, automatic refresh, PDF management, external browser redirection, url string tracking, FontAwesome buttons, Progressbar
' THIS IS NOT MEANT TO REPLACE A SOPHISTICATED BROWSER

'Dependencies :
'- WebViewExtras
'- FontAwesome

'New in version 1.1 :
'- Progressbar (color defined in designer property)
'- Corrected a bug with Home button
'
'As designer property was added, you need to open and save your layouts containing SimpleBrowser

'https://www.b4x.com/android/forum/threads/customview-simplebrowser.75703/#post-481491

'Events :
'	TrackedUrlStringAppears : raised when a tracked string appears in the Url
#Event: TrackedUrlStringAppears(Url as String, Html As String)
'	TrackedUrlStringDisappears : raised when a tracked string disappears in the Url
#Event: TrackedUrlStringDisappears(Url as String, Html As String)
'	PageFinished : raised when a page is totally loaded
#Event: PageFinished(Url as String, Html As String)

'#DesignerProperty: Key: BooleanExample, DisplayName: Boolean Example, FieldType: Boolean, DefaultValue: True, Description: Example of a boolean property.
'#DesignerProperty: Key: IntExample, DisplayName: Int Example, FieldType: Int, DefaultValue: 10, MinRange: 0, MaxRange: 100, Description: Note that MinRange and MaxRange are optional.
'#DesignerProperty: Key: StringWithListExample, DisplayName: String With List, FieldType: String, DefaultValue: Sunday, List: Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday
'#DesignerProperty: Key: StringExample, DisplayName: String Example, FieldType: String, DefaultValue: Text
'#DesignerProperty: Key: ColorExample, DisplayName: Color Example, FieldType: Color, DefaultValue: 0xFFCFDCDC, Description: You can use the built-in color picker to find the color values.
'#DesignerProperty: Key: DefaultColorExample, DisplayName: Default Color Example, FieldType: Color, DefaultValue: Null, Description: Setting the default value to Null means that a nullable field will be displayed.

'Customview designer properties (more informations in the designer)
'	Default_Top_bar_visible : shows or hides the top bar including buttons and url
#DesignerProperty: Key: Default_Top_bar_visible, DisplayName: Top bar visible, FieldType: Boolean, DefaultValue: True, Description: Sets the visibility of the top bar.
'	Default_Auto_refresh_delay : defines an automatic refresh delay
#DesignerProperty: Key: Default_Auto_refresh_delay, DisplayName: Auto refresh delay, FieldType: Int, DefaultValue: 0, MinRange: 0, Description: Sets the delay before auto refresh in seconds. 0 disables auto refresh.
'	Default_tracked_url_string : defines a string which is searched in the url to raise events TrackedUrlStringAppears and TrackedUrlStringDisappears
#DesignerProperty: Key: Default_tracked_url_string, DisplayName: Tracked string, FieldType: String, DefaultValue: login, Description : Sets the url string to be tracked
'	Default_Progressbar_color : defines the color of the progressbar
#DesignerProperty: Key: Default_Progressbar_color, DisplayName: Progressbar color, FieldType: Color, DefaultValue: 0xFFCFDCDC, Description: Sets the color of the progressbar

Sub Class_Globals
	Private EventName As String 
	Private CallBack As Object 
	Private mBase As Panel
    Private Const DefaultColorConstant As Int = -984833 									'ignore
	
	Private FA As FontAwesome	' Icons list is here : http://fontawesome.io/cheatsheet/
	Private Buttons_size As Int
	Private Browser_url_width As Int
	Private Pnl_Top_bar As Panel
	Private B_Home As Button
	Private B_Previous As Button
	Private B_Next As Button
	Private B_Refresh As Button
	Private B_External_browser As Button
	Private Browser_webViewExtras As WebViewExtras
	Private Home_url As String
	Private Loading_box As Label
	Private Refresh_timer As Timer
	
	' Properties
	Public Browser_url As EditText
	Public Browser_progressbar As Label 
	Public Browser_webView As WebView
	Private mAuto_refresh_delay As Long
	Private mTracked_string As String
	Public TrackedPageDisplayed As Boolean
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	EventName = vEventName
	CallBack = vCallback
End Sub

#Region === User interface ====================================================================
Public Sub DesignerCreateView (Base As Panel, Lbl As Label, Props As Map)
	mBase = Base
	' Gets default values from designer ------------------------------
	mAuto_refresh_delay = Props.Get("Default_Auto_refresh_delay") * 1000
	Refresh_timer.Initialize("Refresh_timer", mAuto_refresh_delay)
	If mAuto_refresh_delay > 0 Then Refresh_timer.Enabled = True
	mTracked_string = Props.Get("Default_tracked_url_string")
	' Buttons --------------------------------------------------------
	Buttons_size = Base.Width/26
	FA.Initialize
	Pnl_Top_bar.Initialize("Pnl_Top_bar")
	Base.AddView(Pnl_Top_bar, 0, 0, Base.Width, Buttons_size)
	Pnl_Top_bar.Visible = Props.Get("Default_Top_bar_visible")
	B_Home.Initialize("B_Home")
	Pnl_Top_bar.AddView(B_Home, 0, 0, Buttons_size, Buttons_size)
	B_Home.Typeface = FA.FontAwesomeTypeface
	B_Home.Text = FA.GetFontAwesomeIconByName("fa-home")
	B_Home.TextSize = 20
	B_Previous.Initialize("B_Previous")
	Pnl_Top_bar.AddView(B_Previous, Buttons_size, 0, Buttons_size, Buttons_size)
	B_Previous.Typeface = FA.FontAwesomeTypeface
	B_Previous.Text = FA.GetFontAwesomeIconByName("fa-chevron-left")
	B_Previous.TextSize = 20
	B_Next.Initialize("B_Next")
	Pnl_Top_bar.AddView(B_Next, Buttons_size*2, 0, Buttons_size, Buttons_size)
	B_Next.Typeface = FA.FontAwesomeTypeface
	B_Next.Text = FA.GetFontAwesomeIconByName("fa-chevron-right")
	B_Next.TextSize = 20
	B_Refresh.Initialize("B_Refresh")
	Pnl_Top_bar.AddView(B_Refresh, Buttons_size*3, 0, Buttons_size, Buttons_size)
	B_Refresh.Typeface = FA.FontAwesomeTypeface
	B_Refresh.Text = FA.GetFontAwesomeIconByName("fa-repeat")
	B_Refresh.TextSize = 20
	B_External_browser.Initialize("B_External_browser")
	Pnl_Top_bar.AddView(B_External_browser, Base.Width-Buttons_size-4, 0, Buttons_size, Buttons_size)
	B_External_browser.Typeface = FA.FontAwesomeTypeface
	B_External_browser.Text = FA.GetFontAwesomeIconByName("fa-external-link")
	B_External_browser.TextSize = 20
	' Browser url ------------------------------------------------------
	Browser_url_width = Base.Width-Buttons_size*5
	Browser_url.Initialize("Browser_url")
	Pnl_Top_bar.AddView(Browser_url,  Buttons_size*4, 0, Browser_url_width, Buttons_size)
	Browser_url.TextColor = Colors.Black
	Browser_url.TextSize = 14
	Browser_url.SingleLine = True
	' Progress bar -----------------------------------------------------
	Browser_progressbar.Initialize("Browser_progressbar")
	Pnl_Top_bar.AddView(Browser_progressbar, Buttons_size*4, Buttons_size*0.9, Browser_url_width, Buttons_size*0.1)
	Browser_progressbar.Visible = False
	Browser_progressbar.Color = Props.Get("Default_Progressbar_color")
	Browser_progressbar.BringToFront
	' Loading box ------------------------------------------------------
	Loading_box.Initialize("Loading_box")
	Base.AddView(Loading_box, Base.Width*0.3, Base.Height*0.4, Base.Width*0.4, Base.Height*0.2)
	Loading_box.Text = "Loading ..."
	Loading_box.TextSize = 46
	Loading_box.TextColor = 0xFF000000
	' Browser webview --------------------------------------------------
	Browser_webView.Initialize("Browser_WebView")
	If Pnl_Top_bar.Visible Then
		Base.AddView(Browser_webView, 0, Buttons_size, Base.Width, Base.Height-Buttons_size)
	Else
		Base.AddView(Browser_webView, 0, 0, Base.Width, Base.Height)
	End If
	' Browser webViewExtras --------------------------------------------
	Browser_webViewExtras.addJavascriptInterface(Browser_webView, "B4A")	'add the B4A javascript interface to the WebView
	Browser_webViewExtras.addWebChromeClient(Browser_webView, "Browser_webViewExtras")	'adding a WebChromeClient will log all browser console message to the android log. so any webpage or javascript errors will be logged
End Sub
#End Region

#Region === Functions =========================================================================
Public Sub getBase As Panel
	Return mBase
End Sub

Public Sub Refresh
	Load_url(Browser_webView.Url)
End Sub

Public Sub Clear
	'Browser_webView.LoadHtml("<html><body>Blank screen .................</body></html>")
	Browser_webView.Visible = False
End Sub

Public Sub Load_url(Url As String)
	If Url.Trim <> "" Then
		If Browser_url.text = "" Then Home_url = Url
		Browser_url.text = Url.Trim
		Browser_webView.Visible = False
		Browser_webView.LoadUrl(Url)
	End If
End Sub
#End Region

#Region === Properties ========================================================================
' Top bar visibility
Public Sub getTop_bar_visible As Boolean
	Return Pnl_Top_bar.Visible
End Sub
Public Sub setTop_bar_visible(Visibility As Boolean)
	If Visibility Then Browser_webView.Top = Buttons_size Else Browser_webView.Top = 0
	Pnl_Top_bar.Visible = Visibility
End Sub

' Automatic refresh delay
Public Sub getAuto_refresh_delay As Long
	Return Refresh_timer.Interval
End Sub
Public Sub setAuto_refresh_delay(vAuto_refresh_delay As Long)
	Refresh_timer.Enabled = False
	Refresh_timer.Initialize("Refresh_timer", vAuto_refresh_delay * 1000)
	Refresh_timer.Enabled = (vAuto_refresh_delay > 0)
End Sub

' String which is searched in the url to raise events TrackedUrlStringAppears and TrackedUrlStringDisappears
Public Sub getTracked_string As String
	Return mTracked_string
End Sub
Public Sub setTracked_string(vTracked_string As String)
	mTracked_string = vTracked_string
End Sub
#End Region

#Region === Events ============================================================================
Private Sub B_Home_Click
	Browser_webView.LoadUrl(Home_url)
End Sub

Private Sub B_Previous_Click
	Browser_webView.Back
End Sub

Private Sub B_Next_Click
	Browser_webView.Forward
End Sub

Private Sub B_Refresh_Click
	Refresh
End Sub

Private Sub B_External_browser_Click
	Private Browser_intent As Intent
	Browser_intent.Initialize(Browser_intent.ACTION_VIEW, Browser_url.text)
	StartActivity(Browser_intent)
End Sub

' Event raised when url is validated in the top bar
Private Sub Browser_url_EnterPressed
	Browser_webView.LoadUrl(Browser_url.Text)
End Sub

' Event raised before a link is processed by the browser
Private Sub Browser_WebView_OverrideUrl (Url As String) As Boolean
	If Url.indexof(".pdf")>0 Then
		' https://www.b4x.com/android/forum/threads/cant-download-pdf-file-in-webview.66746/page-2
		' https://www.b4x.com/android/forum/threads/best-way-to-display-pdf-file.17640/#post101055
		' https://www.b4x.com/android/forum/pages/results/?query=open+pdf
		Private Browser_intent As Intent
		Browser_intent.Initialize(Browser_intent.ACTION_VIEW, Url)
		StartActivity(Browser_intent)
		Return True
	Else
		Return False
	End If
End Sub
	
' Event not working for the moment
Public Sub Browser_WebView_UserAndPasswordRequired (Host As String, Realm As String) As String()
	'See this thread : https://www.b4x.com/android/forum/threads/how-to-load-a-url-with-username-password.22006/#post-127922
	'Webview documentation : https://www.b4x.com/android/help/views.html#webview
	' This event is never raised even if it is described in Webview documentation 
	Log("Browser_WebView_UserAndPasswordRequired")
	Return Array As String("someuser", "password123")
End Sub

' Event raised when the page is totally loaded
Public Sub Browser_WebView_PageFinished (Url As String)
	Browser_webView.Visible = True
	Browser_url.text = Url.Trim
	Browser_progressbar.Visible = False
	Dim Javascript As String
	Javascript="B4A.CallSub('SimpleBrowserHTML', false, document.documentElement.outerHTML)"	'see the documentation http://www.basic4ppc.com/forum/additional-libraries-classes-official-updates/12453-webviewextras.html#post70053 for details of the second parameter callUIThread
	Browser_webViewExtras.executeJavascript(Browser_webView, Javascript)
End Sub

' Used with PageFinished event
Private Sub SimpleBrowserHTML(Html As String)										'ignore
	If Browser_url.text.ToLowerCase.Contains(mTracked_string.Trim.ToLowerCase) And Not(TrackedPageDisplayed) Then
		' Raises TrackedUrlStringAppears event
		CallSub3(CallBack, EventName & "_" & "TrackedUrlStringAppears", Browser_webView.Url.Trim, Html)
		TrackedPageDisplayed = True
	End If
	If Not(Browser_url.text.ToLowerCase.Contains(mTracked_string.Trim.ToLowerCase)) And TrackedPageDisplayed Then
		' Raises TrackedUrlStringDisappears event
		CallSub3(CallBack, EventName & "_" & "TrackedUrlStringDisappears", Browser_webView.Url.Trim, Html)
		TrackedPageDisplayed = False
	End If
	'Raises PageFinished event
	CallSub3(CallBack, EventName & "_" & "PageFinished", Browser_webView.Url.Trim, Html)
End Sub

' Event raised when there is a progression while loading the page
Sub Browser_webViewExtras_ProgressChanged(NewProgress As Int)
	Browser_progressbar.Visible = True
	Browser_progressbar.Width = Browser_url_width*NewProgress/100
End Sub

' Event raised when the refresh timer delay is reached
Private Sub Refresh_timer_Tick
	Refresh
End Sub
#End Region